#**************************************************************************//**
#   @file   data_capture.tcl
#   @brief  Initiate a data acquisition and save the data on a .csv file
#   @author Istvan Csomortani (istvan.csomortani@analog.com)
#
#******************************************************************************
# Copyright 2013(c) Analog Devices, Inc.
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#  - Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  - Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#  - Neither the name of Analog Devices, Inc. nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#  - The use of this software may or may not infringe the patent rights
#    of one or more patent holders.  This license does not release you
#    from the requirement that you obtain separate licenses from these
#    patent holders to use this software.
#  - Use of the software either in source or binary form, must be run
#    on or directly connected to an Analog Devices Inc. component.
#
# THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
# AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#******************************************************************************
#   SVN Revision: 1984
#*****************************************************************************/

################################################################################
##                              CONVERTER SETUP                              ##
###############################################################################
## Programmable Gain Stage - possible values : 1, 2, 3, 4, 6, 8, 12,
##                                             16, 24, 32, 48, 64, 96, 128
## Power-Down Modes :
##      |PD2 |PD1 |PD0 |COMPA/B |COMPC/D |ADC A/B |
##      | 0  | 0  | 0  |  OFF   |  OFF   |  OFF   |
##      | 0  | 0  | 1  |  OFF   |  OFF   |  ON    |
##      | 0  | 1  | 0  |  OFF   |  ON    |  OFF   |
##      | 0  | 1  | 1  |  ON    |  OFF   |  OFF   |
##      | 1  | 0  | 0  |  ON    |  ON    |  OFF   |
##      | 1  | 0  | 1  |  ON    |  ON    |  ON    |
##      | 1  | 1  | 0  |  OFF   |  OFF   |  OFF   |
###############################################################################

set gain 2
set powermodes 5

# end of configuration register setup

puts "Connecting to device ..."
set mm [ lindex [ get_service_paths master ] 0 ]
close_service master $mm
open_service master $mm

# Data acquisition
puts "Acquiring data ..."
set startacq 0

# Set up the configuration register
master_write_32 $mm 0x4a0018 $gain
master_write_32 $mm 0x4a001C $powermodes

after 1000

puts "Starting acquisition ..."
set startacq [master_read_32 $mm 0x4a0000 1]
set point [master_read_32 $mm 0x4a0014 1]

master_write_32 $mm $point 1

while {$startacq == 0} {
    set startacq [master_read_32 $mm 0x4a0000 1]
}

# Maximum number of data bytes that can be acquired is 1048576
set data [ master_read_memory $mm 0x4e0000 524288 ]

puts "Converting acquired data to 12 bit values"
for { set i 0 } { $i < [ llength $data ] } { incr i 4} {
    set a [ lindex $data $i ]
    set b [ lindex $data [ expr { $i + 1 } ] ]
    set c [ lindex $data [ expr { $i + 2 } ] ]
    set d [ lindex $data [ expr { $i + 3 } ] ]
    set value 0
    set value [expr $value + [ expr $a ] ]
    set value [expr $value + [ expr $b << 8] ]
    lappend newData [ expr $value ]
    set value 0
    set value [expr $value + [ expr $c ] ]
    set value [expr $value + [ expr $d << 8] ]
    lappend newData [ expr $value ]
}

puts "Processing 12 bit data as two's complement"
set final " "
for { set i 0 } { $i < [ llength $newData ] } { incr i 2} {

    set a [ lindex $newData $i ]
    set decimal [expr $a]
    if { $decimal > 2048 } {
        set decimal [expr $decimal - 4096 ]
    }
    lappend final [expr $decimal],

    set a [ lindex $newData $i+1]
    set decimal [expr $a]
    if { $decimal > 2048 } {
        set decimal [expr $decimal - 4096 ]
    }
    lappend final [expr $decimal]\n
}

unset data
unset newData

puts "Moving data into file Acquisition.csv"
# If the file is allready exist it will be overwritten, save the previouse
# acquisition result if needed

set openFile 0

while { $openFile == 0 } {
    if [catch { open "Acquisition.csv" "w+" } fid ] {
        puts stderr "Error opening the file. Please make sure the file is not \
            opened by another program"
        after 1000
        puts "Trying again..."
    } else {
        set fd $fid
        set openFile 1
        puts "File opened"
    }
}

puts -nonewline $fd [ join $final ]
close $fd

master_write_32 $mm 0x4a000C 1
unset final
unset point
unset startacq

puts "Iteration complete ..."
close_service master $mm
