// -----------------------------------------------------------------------------
//
// Copyright 2013(c) Analog Devices, Inc.
//
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//  - Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//  - Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in
//    the documentation and/or other materials provided with the
//    distribution.
//  - Neither the name of Analog Devices, Inc. nor the names of its
//    contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//  - The use of this software may or may not infringe the patent rights
//    of one or more patent holders.  This license does not release you
//    from the requirement that you obtain separate licenses from these
//    patent holders to use this software.
//  - Use of the software either in source or binary form, must be run
//    on or directly connected to an Analog Devices Inc. component.
//
// THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
// IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// -----------------------------------------------------------------------------
// FILE NAME : AD7262.v
// MODULE NAME : AD7262
// AUTHOR : Istvan Csomortani
// AUTHOR’S EMAIL : istvan.csomortani@analog.com
// -----------------------------------------------------------------------------
// SVN REVISION: 1984
// -----------------------------------------------------------------------------
// KEYWORDS : AD7262
// -----------------------------------------------------------------------------
// PURPOSE : Driver for the AD7262  12-Bit, 2-channel ADC True differential analog inputs
// Throughput rate 1MSPS with a 73db SNR
//
// -----------------------------------------------------------------------------
// REUSE ISSUES
// Reset Strategy      : Active low reset signal
// Clock Domains       : 2 clocks   sys clock 48 Mhz
//                                  sclk clock 24 Mhz
// Critical Timing     : N/A
// Test Features       : N/A
// Asynchronous I/F    : N/A
// Instantiations      : N/A
// Synthesizable (y/n) : Y
// Target Device       : AD7262
// Other               : The driver is intended to be used for AD7262 ADCs configuration
// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------

`timescale 1 ns/ 100 ps

//------------------------------------------------------------------------------
//--------------------------- Module Declaration -------------------------------
//------------------------------------------------------------------------------

module AD7262 (
   // Sys Clock
   input                fpga_clk_i,
   // SPI Clock
   input                adc_clk_i,
   input                rst_n_i,

   // Data interface
   input                data_wr_en_i,
   output reg   [11:0]  data_a_o,
   output reg   [11:0]  data_b_o,
   output               data_rd_en_o,

   // Interface with the ADC
   output               adc_sclk_o,
   output               adc_cs_n_o,
   input                adc_douta_i,
   input                adc_doutb_i,
   //Debug
   output       [3:0]   adc_state_o

   );

//------------------------------------------------------------------------------
//----------------------- Registers/Wires Declaration ---------------------------------
//------------------------------------------------------------------------------
// State registers
reg [3:0]   adc_state;
reg [3:0]   adc_state_ns;
// Enable bit for the SCLK
reg         adc_sclk_en;
reg [31:0]  data_a_buffer;
reg [31:0]  data_b_buffer;
reg [7:0]   adc_sclk_cnt;
reg [7:0]   adc_quiet_cnt;
reg         adc_cs_n_s;
reg         data_rd_en_s;
reg         data_rd_en_ss;

wire        adc_sclk_s;

//------------------------------------------------------------------------------
//--------------------------- Local Parameters ---------------------------------
//------------------------------------------------------------------------------
// SPI control
parameter   ADC_SCLK_CNT    = 8'd30;
parameter   ADC_QUIET_CNT   = 8'd1;
// FSM states
parameter   IDLE            = 4'b0001;
parameter   START           = 4'b0010;
parameter   TRANS           = 4'b0100;
parameter   QUIET           = 4'b1000;

//------------------------------------------------------------------------------
//------------------------ Assign/Always Blocks --------------------------------
//------------------------------------------------------------------------------

assign adc_sclk_s       = (adc_sclk_en == 1'b1) ? adc_clk_i : 1'b1;
assign adc_sclk_o       = adc_sclk_s;
assign adc_cs_n_o       = adc_cs_n_s;
assign adc_state_o      = adc_state;
assign data_rd_en_o     = ((data_rd_en_ss == 1'b1) && (data_rd_en_s == 1'b0)) ? 1'b1 : 1'b0;

// Data output latch
always @(posedge fpga_clk_i)
begin
    if(rst_n_i == 1'b0)
    begin
        data_a_o <= 12'b0;
        data_b_o <= 12'b0;
    end
    else if(data_rd_en_s == 1'b1)
    begin
        data_b_o <= data_b_buffer[11:0];
        data_a_o <= data_a_buffer[11:0];
    end
end

// Sync the data_rd_en to the system clock
always @(posedge fpga_clk_i)
begin
    data_rd_en_ss   <= data_rd_en_s;
end

// State register and SPI control logic
always @(posedge adc_clk_i)
begin
    if(rst_n_i == 1'b0)
    begin
        adc_state       <= IDLE;
        adc_sclk_cnt    <= ADC_SCLK_CNT;
        adc_quiet_cnt   <= ADC_QUIET_CNT;
    end
    else
    begin
        adc_state <= adc_state_ns;
        case(adc_state)
            IDLE :
            begin
                adc_cs_n_s      <= 1'b1;
                adc_sclk_en     <= 1'b0;
                data_rd_en_s    <= 1'b0;
                adc_sclk_cnt    <= ADC_SCLK_CNT;
                adc_quiet_cnt   <= ADC_QUIET_CNT;
            end
            START :
            begin
                adc_cs_n_s      <= 1'b0;
                adc_sclk_en     <= 1'b0;
                data_rd_en_s    <= 1'b0;
            end
            TRANS :
            begin
                adc_cs_n_s      <= 1'b0;
                adc_sclk_cnt    <= adc_sclk_cnt - 8'h1;
                adc_sclk_en     <= 1'b1;
                data_rd_en_s    <= 1'b0;
            end
            QUIET :
            begin
                adc_quiet_cnt   <= adc_quiet_cnt - 8'h1;
                adc_cs_n_s      <= 1'b0;
                adc_sclk_en     <= 1'b0;
                data_rd_en_s    <= 1'b1;
            end
        endcase
    end
end

// Shifting in the data from SPI data bus
// The internal data buffers will be reseted in case of an active reset signal,
// or after every transaction
always @(negedge adc_sclk_s)
begin
    if(rst_n_i == 1'b0)
    begin
        data_a_buffer <= 32'h0;
        data_b_buffer <= 32'h0;
    end
    else
    begin
        data_a_buffer <= {data_a_buffer[30:0], adc_douta_i};
        data_b_buffer <= {data_b_buffer[30:0], adc_doutb_i};
    end
end

// Next state logic
always @(adc_state, rst_n_i, data_wr_en_i, adc_sclk_cnt)
begin
    if(rst_n_i == 1'b0)
    begin
        adc_state_ns = IDLE;
    end
    else
    begin
        case(adc_state)
            IDLE :
            begin
                if(data_wr_en_i == 1'b1)
                begin
                    adc_state_ns = START;
                end
                else
                begin
                    adc_state_ns = IDLE;
                end
            end
            START :
            begin
                adc_state_ns = TRANS;
            end
            TRANS :
            begin
                if(adc_sclk_cnt == 8'h0)
                begin
                    adc_state_ns = QUIET;
                end
                else
                begin
                    adc_state_ns = TRANS;
                end
            end
            QUIET :
                if(adc_quiet_cnt == 8'h0)
                begin
                    adc_state_ns = IDLE;
                end
                else
                begin
                    adc_state_ns = QUIET;
                end
            default :
            begin
                adc_state_ns = IDLE;
            end
        endcase
    end
end

endmodule
