/******************************************************************************
*   @file   ad7262.c
*   @brief  Driver for the AD7262 ADC altera avalon IP
*   @author Istvan Csomortani
*   @author's email istvan.csomortani@analog.com
*
*******************************************************************************
* Copyright 2013(c) Analog Devices, Inc.
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in
*    the documentation and/or other materials provided with the
*    distribution.
*  - Neither the name of Analog Devices, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*  - The use of this software may or may not infringe the patent rights
*    of one or more patent holders.  This license does not release you
*    from the requirement that you obtain separate licenses from these
*    patent holders to use this software.
*  - Use of the software either in source or binary form, must be run
*    on or directly connected to an Analog Devices Inc. component.
*
* THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*******************************************************************************
*   SVN Revision: 1984
******************************************************************************/

/*****************************************************************************/
/***************************** Include Files *********************************/
/*****************************************************************************/

#include "system.h"
#include "altera_avalon_pio_regs.h"
#include "ad7262.h"
#include "ad7262_regs.h"

/*****************************************************************************/
/************************ Functions Definitions ******************************/
/*****************************************************************************/

/****************************************************************
* @brief This function performs a softare reset on the AD7262
*       core component
*
* @return none
*****************************************************************/
void AD7262_SwReset(void)
{
    unsigned int controlreg = IORD_AD7262_CONTROL(AD7262_0_BASE);
    IOWR_AD7262_CONTROL(AD7262_0_BASE, controlreg | (1 << SW_RESET));
}

/****************************************************************
* @brief This function performs a harware reset on the AD7262
*       chip, and after that enable the ADC A and ADC B lines
*
* @return none
*****************************************************************/
void AD7262_HwReset(void)
{
    IOWR_ALTERA_AVALON_PIO_SET_BITS(PWR_MODES_BASE, 7);
    usleep(50);
    IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(PWR_MODES_BASE, 7);
}

/****************************************************************
* @brief Initiate an internal calibration of the converter
*
* parameter     None
*
* @return There is no return value
*****************************************************************/

void AD7262_Calibrate(void)
{
    unsigned int controlreg = IORD_AD7262_CONTROL(AD7262_0_BASE);

    IOWR_ALTERA_AVALON_PIO_SET_BITS(CAL_BASE, 1);
    IOWR_AD7262_CONTROL (AD7262_0_BASE, controlreg | (1 << WR_DATA));
    IOWR_AD7262_CONTROL (AD7262_0_BASE, controlreg | (1 << WR_DATA));
    IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(CAL_BASE, 1);
}

/****************************************************************
* @brief This function activates the TestMode feature of the HDL core
*
*
* @return none
*****************************************************************/
void AD7262_SetTestMode(void)
{
    unsigned int controlreg = IORD_AD7262_CONTROL(AD7262_0_BASE);
    IOWR_AD7262_CONTROL(AD7262_0_BASE, controlreg | (1 << TEST_MODE));
}

/****************************************************************
* @brief Updates the selected internal register on the ADC converter
*
* parameter reg             Selects the register in which the value should be
*                   written
* parameter value           The new value to be written in the configuration
*                   register
*
* @return There is no return value
*****************************************************************/

void AD7262_WriteEvalBoardRegister(unsigned char reg, unsigned int value)
{
    unsigned int data = ((reg & 0xF) << 12) + (value & 0xFFF);
    unsigned int controlreg = IORD_AD7262_CONTROL(AD7262_0_BASE);

    IOWR_AD7262_DUT_WR(AD7262_0_BASE, data);
    IOWR_AD7262_CONTROL (AD7262_0_BASE, controlreg | (1 << WR_DATA));
}

/****************************************************************
* @brief Updates the selected internal register on the ADC converter
*
* parameter reg             Selects the register from which the value should be
*                   read
*
* @return Return the value of the specified register
*****************************************************************/

unsigned int AD7262_ReadEvalBoardRegister(unsigned char reg)
{
    unsigned int data = (0x1 << 12) + ((reg & 0xF) << 8);
    unsigned int controlreg = IORD_AD7262_CONTROL(AD7262_0_BASE);

    IOWR_AD7262_DUT_WR(AD7262_0_BASE, data);
    IOWR_AD7262_CONTROL (AD7262_0_BASE, controlreg | (1 << WR_DATA));

return IORD_AD7262_DUT_RD(AD7262_0_BASE);
}

/****************************************************************
* @brief Setup the programmable gain stage of the converter
*
* parameter gain        Possible values of the PGA : 1, 2, 3, 4, 6, 8, 12,
*                           16, 24, 32, 48, 64, 96, 128
*
* @return There is no return value
*****************************************************************/

void AD7262_SetGain(unsigned int gain)
{
    IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(GAIN_SELECT_BASE, 0xF);
    switch(gain)
    {
        case 0:
            IOWR_ALTERA_AVALON_PIO_SET_BITS(GAIN_SELECT_BASE, 0x0);
            break;
        case 1:
            IOWR_ALTERA_AVALON_PIO_SET_BITS(GAIN_SELECT_BASE, 0x1);
            break;
        case 2:
            IOWR_ALTERA_AVALON_PIO_SET_BITS(GAIN_SELECT_BASE, 0x2);
            break;
        case 3:
            IOWR_ALTERA_AVALON_PIO_SET_BITS(GAIN_SELECT_BASE, 0x3);
            break;
        case 4:
            IOWR_ALTERA_AVALON_PIO_SET_BITS(GAIN_SELECT_BASE, 0x4);
            break;
        case 6:
            IOWR_ALTERA_AVALON_PIO_SET_BITS(GAIN_SELECT_BASE, 0x5);
            break;
        case 8:
            IOWR_ALTERA_AVALON_PIO_SET_BITS(GAIN_SELECT_BASE, 0x6);
            break;
        case 12:
            IOWR_ALTERA_AVALON_PIO_SET_BITS(GAIN_SELECT_BASE, 0x7);
            break;
        case 16:
            IOWR_ALTERA_AVALON_PIO_SET_BITS(GAIN_SELECT_BASE, 0x8);
            break;
        case 24:
            IOWR_ALTERA_AVALON_PIO_SET_BITS(GAIN_SELECT_BASE, 0x9);
            break;
        case 32:
            IOWR_ALTERA_AVALON_PIO_SET_BITS(GAIN_SELECT_BASE, 0xA);
            break;
        case 48:
            IOWR_ALTERA_AVALON_PIO_SET_BITS(GAIN_SELECT_BASE, 0xB);
            break;
        case 64:
            IOWR_ALTERA_AVALON_PIO_SET_BITS(GAIN_SELECT_BASE, 0xC);
            break;
        case 96:
            IOWR_ALTERA_AVALON_PIO_SET_BITS(GAIN_SELECT_BASE, 0xD);
            break;
        case 128:
            IOWR_ALTERA_AVALON_PIO_SET_BITS(GAIN_SELECT_BASE, 0xE);
            break;
        default :
            IOWR_ALTERA_AVALON_PIO_SET_BITS(GAIN_SELECT_BASE, 0x2);
            break;
    }
}

/****************************************************************
* @brief Setup the power-down modes of the converter
*
* parameter powermode  |PD2 |PD1 |PD0 |COMPA/B |COMPC/D |ADC A/B |
*                      | 0  | 0  | 0  |  OFF   |  OFF   |  OFF   |
*                      | 0  | 0  | 1  |  OFF   |  OFF   |  ON    |
*                      | 0  | 1  | 0  |  OFF   |  ON    |  OFF   |
*                      | 0  | 1  | 1  |  ON    |  OFF   |  OFF   |
*                      | 1  | 0  | 0  |  ON    |  ON    |  OFF   |
*                      | 1  | 0  | 1  |  ON    |  ON    |  ON    |
*                      | 1  | 1  | 0  |  OFF   |  OFF   |  OFF   |
* @return There is no return value
*****************************************************************/
void AD7262_SetPowerMode(unsigned int powermode)
{
    IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(PWR_MODES_BASE, 0x7);
    IOWR_ALTERA_AVALON_PIO_SET_BITS(PWR_MODES_BASE, (powermode & 0x7));
}

/****************************************************************
* @brief This function performs an initialization on the AD7262
*       configuration pins
*
* @return none
*****************************************************************/
void AD7262_InitPins(void)
{
    AD7262_SetGain(2);
    AD7262_SetPowerMode(5);
}

/****************************************************************
* @brief Starts the data acquisition process
* parameter size will send the number of samples that need to
*                be acquired. For CED1Z must be under 512*1024
* parameter destination will point to the memory location starting
*                       from where data will be written
*
* @return 0 if acquisition starts successfully
* @return 1 if the number of samples is too large
*****************************************************************/
int AD7262_StartAcquisition(int size, unsigned int destination)
{
    unsigned int controlreg = IORD_AD7262_CONTROL(AD7262_0_BASE);

    if ( size > (MEMORY_LINE_SIZE * MAX_MEMORY_LINES))
    {
        return 1;
    }
    else
    {
        if (size % 2)
        {
            size++;
        }
        IOWR_AD7262_ACQ_COUNT(AD7262_0_BASE, size);
        IOWR_AD7262_BASE(AD7262_0_BASE, destination);
        IOWR_AD7262_CONTROL(AD7262_0_BASE, controlreg | (1 << START_ACQ));
    }
    return 0;
}

/****************************************************************
* @brief Reads the status register
*
* @return The status register
*****************************************************************/
int AD7262_ReadStatus (void)
{
    return IORD_AD7262_STATUS(AD7262_0_BASE);
}

/****************************************************************
* @brief Reads the state register of the internal FSM
*
* @return State register RD_FSM
*****************************************************************/
int AD7262_ReadState (void)
{
    return (IORD_AD7262_DUT_STATE(AD7262_0_BASE));
}
